@@ -1,7 +1,7 @@
 from math import ceil
 
 from sbws.globals import (fail_hard, SBWS_SCALE_CONSTANT, TORFLOW_SCALING,
-                          SBWS_SCALING, TORFLOW_BW_MARGIN, TORFLOW_ROUND_DIG,
+                          SBWS_SCALING, TORFLOW_BW_MARGIN, PROP276_ROUND_DIG,
                           DAY_SECS, NUM_MIN_RESULTS)
 from sbws.lib.v3bwfile import V3BWFile
 from sbws.lib.resultdump import load_recent_results_in_datadir
@@ -9,6 +9,7 @@ from argparse import ArgumentDefaultsHelpFormatter
 import os
 import logging
 from sbws.util.timestamp import now_fname
+from sbws.lib import destination
 
 log = logging.getLogger(__name__)
 
@@ -22,7 +23,9 @@ def gen_parser(sub):
         '(latest.v3bw is an atomically created symlink in the same '\
         'directory as output.) '\
         'If the file is transferred to another host, it should be written to '\
-        'a temporary path, then renamed to the V3BandwidthsFile path.'
+        'a temporary path, then renamed to the V3BandwidthsFile path.\n'\
+        'The default scaling method is torflow\'s one. To use different'\
+        'scaling methods or no scaling, see the options.'
     p = sub.add_parser('generate', description=d,
                        formatter_class=ArgumentDefaultsHelpFormatter)
     p.add_argument('--output', default=None, type=str,
@@ -40,20 +43,21 @@ def gen_parser(sub):
                    'are, but scale them such that we have a budget of '
                    'scale_constant * num_measured_relays = bandwidth to give '
                    'out, and we do so proportionally')
-    p.add_argument('-t', '--scale-torflow', action='store_const',
-                   default=True, const=False,
-                   help='If specified, do not use bandwidth values as they '
-                   'are, but scale them in the way Torflow does.')
+    p.add_argument('-t', '--scale-torflow', action='store_true',
+                   default=True,
+                   help='If specified, scale measurements using torflow\'s '
+                   'method. This option is kept for compatibility with older '
+                   'versions and it is silently ignored, since it is the '
+                   'default.')
     p.add_argument('-w', '--raw', action='store_true',
                    help='If specified, do use bandwidth raw measurements '
                    'without any scaling.')
     p.add_argument('-m', '--torflow-bw-margin', default=TORFLOW_BW_MARGIN,
                    type=float,
                    help="Cap maximum bw when scaling as Torflow. ")
-    p.add_argument('-r', '--torflow-round-digs', default=TORFLOW_ROUND_DIG,
-                   type=int,
-                   help="Number of most significant digits to round bw "
-                        "when scaling as Torflow.")
+    p.add_argument('-r', '--round-digs', '--torflow-round-digs',
+                   default=PROP276_ROUND_DIG, type=int,
+                   help="Number of most significant digits to round bw.")
     p.add_argument('-p', '--secs-recent', default=None, type=int,
                    help="How many secs in the past are results being "
                         "still considered. Note this value will supersede "
@@ -63,6 +67,7 @@ def gen_parser(sub):
                         "other.")
     p.add_argument('-n', '--min-num', default=NUM_MIN_RESULTS, type=int,
                    help="Mininum number of a results to consider them.")
+    return p
 
 
 def main(args, conf):
@@ -80,6 +85,8 @@ def main(args, conf):
     elif args.raw:
         scaling_method = None
     else:
+        # sbws will scale as torflow until we have a better algorithm for
+        # scaling (#XXX)
         scaling_method = TORFLOW_SCALING
     if args.secs_recent:
         fresh_days = ceil(args.secs_recent / 24 / 60 / 60)
@@ -88,7 +95,7 @@ def main(args, conf):
     reset_bw_ipv4_changes = conf.getboolean('general', 'reset_bw_ipv4_changes')
     reset_bw_ipv6_changes = conf.getboolean('general', 'reset_bw_ipv6_changes')
     results = load_recent_results_in_datadir(
-        fresh_days, datadir, success_only=True,
+        fresh_days, datadir,
         on_changed_ipv4=reset_bw_ipv4_changes,
         on_changed_ipv6=reset_bw_ipv6_changes)
     if len(results) < 1:
@@ -98,10 +105,14 @@ def main(args, conf):
     state_fpath = conf.getpath('paths', 'state_fname')
     consensus_path = os.path.join(conf.getpath('tor', 'datadir'),
                                   "cached-consensus")
-    bw_file = V3BWFile.from_results(results, state_fpath, args.scale_constant,
-                                    scaling_method,
+    # Accept None as scanner_country to be compatible with older versions.
+    scanner_country = conf['scanner'].get('country')
+    destinations_countries = destination.parse_destinations_countries(conf)
+    bw_file = V3BWFile.from_results(results, scanner_country,
+                                    destinations_countries, state_fpath,
+                                    args.scale_constant, scaling_method,
                                     torflow_cap=args.torflow_bw_margin,
-                                    torflow_round_digs=args.torflow_round_digs,
+                                    round_digs=args.round_digs,
                                     secs_recent=args.secs_recent,
                                     secs_away=args.secs_away,
                                     min_num=args.min_num,
