@@ -4,7 +4,6 @@ import time
 import logging
 from glob import glob
 from threading import Thread
-from threading import Event
 from threading import RLock
 from queue import Queue
 from queue import Empty
@@ -14,6 +13,7 @@ from enum import Enum
 from sbws.globals import RESULT_VERSION, fail_hard
 from sbws.util.filelock import DirectoryLock
 from sbws.lib.relaylist import Relay
+from .. import settings
 
 log = logging.getLogger(__name__)
 
@@ -133,6 +133,8 @@ def load_recent_results_in_datadir(fresh_days, datadir, success_only=False,
     Results as a list '''
     assert isinstance(fresh_days, int)
     assert os.path.isdir(datadir)
+    # Inform the results are being loaded, since it takes some seconds.
+    log.info("Reading and processing previous measurements.")
     results = {}
     today = datetime.utcfromtimestamp(time.time())
     data_period = fresh_days + 2
@@ -190,29 +192,82 @@ class _ResultType(_StrEnum):
     ErrorCircuit = 'error-circ'
     ErrorStream = 'error-stream'
     ErrorAuth = 'error-auth'
+    # When it can not be found a second relay suitable to measure a relay.
+    # It is used in ``ResultErrorSecondRelay``.
+    ErrorSecondRelay = 'error-second-relay'
+    # When there is not a working destination Web Server.
+    # It is used in ``ResultErrorDestionation``.
+    ErrorDestination = 'error-destination'
 
 
 class Result:
-    ''' A simple struct to pack a measurement result into so that other code
-    can be confident it is handling a well-formed result. '''
+    """A bandwidth measurement for a relay.
+
+    It re-implements :class:`~sbws.lib.relaylist.Relay` as a inner class.
+    """
 
     class Relay:
-        ''' Implements just enough of a stem RouterStatusEntryV3 for this
-        Result class to be happy '''
+        """A Tor relay.
+
+        It re-implements :class:`~sbws.lib.relaylist.Relay`
+        with the attributes needed.
+
+        .. note:: in a future refactor it would be simpler if a ``Relay`` has
+           measurements and a measurement has a relay,
+           instead of every measurement re-implementing ``Relay``.
+        """
         def __init__(self, fingerprint, nickname, address, master_key_ed25519,
-                     average_bandwidth=None, observed_bandwidth=None):
+                     average_bandwidth=None, burst_bandwidth=None,
+                     observed_bandwidth=None, consensus_bandwidth=None,
+                     consensus_bandwidth_is_unmeasured=None,
+                     # Counters to be stored by relay and not per measurement,
+                     # since the measurements might fail.
+                     relay_in_recent_consensus_count=None,
+                     relay_recent_measurement_attempt_count=None,
+                     relay_recent_priority_list_count=None):
+            """
+            Initializes a ``Result.Relay``.
+
+            .. note:: in a future refactor the attributes should be dinamic
+               to easy adding/removing them.
+               They are shared by  :class:`~sbws.lib.relaylist.Relay` and
+               :class:`~sbws.lib.v3bwfile.V3BWLine` and there should not be
+               repeated in every class.
+            """
             self.fingerprint = fingerprint
             self.nickname = nickname
             self.address = address
             self.master_key_ed25519 = master_key_ed25519
             self.average_bandwidth = average_bandwidth
+            self.burst_bandwidth = burst_bandwidth
             self.observed_bandwidth = observed_bandwidth
+            self.consensus_bandwidth = consensus_bandwidth
+            self.consensus_bandwidth_is_unmeasured = \
+                consensus_bandwidth_is_unmeasured
+            self.relay_in_recent_consensus_count = \
+                relay_in_recent_consensus_count
+            self.relay_recent_measurement_attempt_count = \
+                relay_recent_measurement_attempt_count
+            self.relay_recent_priority_list_count = \
+                relay_recent_priority_list_count
 
-    def __init__(self, relay, circ, dest_url, scanner_nick, t=None):
-        self._relay = Result.Relay(relay.fingerprint, relay.nickname,
-                                   relay.address, relay.master_key_ed25519,
-                                   relay.average_bandwidth,
-                                   relay.observed_bandwidth)
+    def __init__(self, relay, circ, dest_url, scanner_nick, t=None,
+                 relay_in_recent_consensus_count=None):
+        """
+        Initilizes the measurement and the relay with all the relay attributes.
+        """
+        self._relay = Result.Relay(
+            relay.fingerprint, relay.nickname,
+            relay.address, relay.master_key_ed25519,
+            relay.average_bandwidth,
+            relay.burst_bandwidth,
+            relay.observed_bandwidth,
+            relay.consensus_bandwidth,
+            relay.consensus_bandwidth_is_unmeasured,
+            relay.relay_in_recent_consensus_count,
+            relay.relay_recent_measurement_attempt_count,
+            relay.relay_recent_priority_list_count
+            )
         self._circ = circ
         self._dest_url = dest_url
         self._scanner = scanner_nick
@@ -226,10 +281,22 @@ class Result:
     def relay_average_bandwidth(self):
         return self._relay.average_bandwidth
 
+    @property
+    def relay_burst_bandwidth(self):
+        return self._relay.burst_bandwidth
+
     @property
     def relay_observed_bandwidth(self):
         return self._relay.observed_bandwidth
 
+    @property
+    def consensus_bandwidth(self):
+        return self._relay.consensus_bandwidth
+
+    @property
+    def consensus_bandwidth_is_unmeasured(self):
+        return self._relay.consensus_bandwidth_is_unmeasured
+
     @property
     def fingerprint(self):
         return self._relay.fingerprint
@@ -246,6 +313,29 @@ class Result:
     def master_key_ed25519(self):
         return self._relay.master_key_ed25519
 
+    @property
+    def relay_in_recent_consensus_count(self):
+        """Number of times the relay was in a consensus."""
+        return self._relay.relay_in_recent_consensus_count
+
+    @property
+    def relay_recent_measurement_attempt_count(self):
+        """Returns the relay recent measurements attemps.
+
+        It is initialized in :class:`~sbws.lib.relaylist.Relay` and
+        incremented in :func:`~sbws.core.scanner.main_loop`.
+        """
+        return self._relay.relay_recent_measurement_attempt_count
+
+    @property
+    def relay_recent_priority_list_count(self):
+        """Returns the relay recent "prioritization"s to be measured.
+
+        It is initialized in :class:`~sbws.lib.relaylist.Relay` and
+        incremented in :func:`~sbws.core.scanner.main_loop`.
+        """
+        return self._relay.relay_recent_priority_list_count
+
     @property
     def circ(self):
         return self._circ
@@ -278,15 +368,31 @@ class Result:
             'type': self.type,
             'scanner': self.scanner,
             'version': self.version,
+            'relay_in_recent_consensus_count':
+                self.relay_in_recent_consensus_count,
+            'relay_recent_measurement_attempt_count':
+                self.relay_recent_measurement_attempt_count,
+            'relay_recent_priority_list_count':
+                self.relay_recent_priority_list_count,
         }
 
     @staticmethod
     def from_dict(d):
-        ''' Given a dict, returns the Result* subtype that is represented by
-        the dict. If we don't know how to parse the dict into a Result and it's
-        likely because the programmer forgot to implement something, raises
-        NotImplementedError. If we can't parse the dict for some other reason,
-        return None. '''
+        """
+        Returns a :class:`~sbws.lib.resultdump.Result` subclass from a
+        dictionary.
+
+        Returns None if the ``version`` attribute is not
+        :const:`~sbws.globals.RESULT_VERSION`
+
+        It raises ``NotImplementedError`` when the dictionary ``type`` can not
+        be parsed.
+
+        .. note:: in a future refactor, the conversions to/from
+           object-dictionary will be simpler using ``setattr`` and ``__dict__``
+
+           ``version`` is not being used and should be removed.
+        """
         assert 'version' in d
         if d['version'] != RESULT_VERSION:
             return None
@@ -301,6 +407,10 @@ class Result:
             return ResultErrorStream.from_dict(d)
         elif d['type'] == _ResultType.ErrorAuth.value:
             return ResultErrorAuth.from_dict(d)
+        elif d['type'] == _ResultType.ErrorSecondRelay.value:
+            return ResultErrorSecondRelay.from_dict(d)
+        elif d['type'] == _ResultType.ErrorDestination.value:
+            return ResultErrorDestination.from_dict(d)
         else:
             raise NotImplementedError(
                 'Unknown result type {}'.format(d['type']))
@@ -345,7 +455,14 @@ class ResultError(Result):
         return ResultError(
             Result.Relay(
                 d['fingerprint'], d['nickname'], d['address'],
-                d['master_key_ed25519']),
+                d['master_key_ed25519'],
+                relay_in_recent_consensus_count=  # noqa
+                    d.get('relay_in_recent_consensus_count', None),  # noqa
+                relay_recent_measurement_attempt_count=  # noqa
+                    d.get('relay_recent_measurement_attempt_count', None),  # noqa
+                relay_recent_priority_list_count=  # noqa
+                    d.get('relay_recent_priority_list_count', None),  # noqa
+                ),
             d['circ'], d['dest_url'], d['scanner'],
             msg=d['msg'], t=d['time'])
 
@@ -386,7 +503,14 @@ class ResultErrorCircuit(ResultError):
         return ResultErrorCircuit(
             Result.Relay(
                 d['fingerprint'], d['nickname'], d['address'],
-                d['master_key_ed25519']),
+                d['master_key_ed25519'],
+                relay_in_recent_consensus_count=  # noqa
+                    d.get('relay_in_recent_consensus_count', None),  # noqa
+                relay_recent_measurement_attempt_count=  # noqa
+                    d.get('relay_recent_measurement_attempt_count', None),  # noqa
+                relay_recent_priority_list_count=  # noqa
+                    d.get('relay_recent_priority_list_count', None),  # noqa
+                ),
             d['circ'], d['dest_url'], d['scanner'],
             msg=d['msg'], t=d['time'])
 
@@ -407,6 +531,90 @@ class ResultErrorStream(ResultError):
     def from_dict(d):
         assert isinstance(d, dict)
         return ResultErrorStream(
+            Result.Relay(
+                d['fingerprint'], d['nickname'], d['address'],
+                d['master_key_ed25519'],
+                relay_in_recent_consensus_count=  # noqa
+                    d.get('relay_in_recent_consensus_count', None),  # noqa
+                relay_recent_measurement_attempt_count=  # noqa
+                    d.get('relay_recent_measurement_attempt_count', None),  # noqa
+                relay_recent_priority_list_count=  # noqa
+                    d.get('relay_recent_priority_list_count', None),  # noqa
+                ),
+            d['circ'], d['dest_url'], d['scanner'],
+            msg=d['msg'], t=d['time'])
+
+    def to_dict(self):
+        d = super().to_dict()
+        return d
+
+
+class ResultErrorSecondRelay(ResultError):
+    """
+    Error when it could not be found a second relay suitable to measure
+    a relay.
+
+    A second suitable relay is a relay that:
+    - Has at least equal bandwidth as the relay to measure.
+    - If the relay to measure is not an exit,
+      the second relay is an exit without `bad` flag and can exit to port 443.
+    - If the relay to measure is an exit, the second relay is not an exit.
+
+    It is instanciated in :func:`~sbws.core.scanner.measure_relay`.
+
+    .. note:: this duplicates code and add more tech-debt,
+       since it's the same as the other
+       :class:`~sbws.lib.resultdump.ResultError` classes except for the
+       ``type``.
+       In a future refactor, there should be only one ``ResultError`` class
+       and assign the type in the ``scanner`` module.
+    """
+    def __init__(self, *a, **kw):
+        super().__init__(*a, **kw)
+
+    @property
+    def type(self):
+        return _ResultType.ErrorSecondRelay
+
+    @staticmethod
+    def from_dict(d):
+        assert isinstance(d, dict)
+        return ResultErrorSecondRelay(
+            Result.Relay(
+                d['fingerprint'], d['nickname'], d['address'],
+                d['master_key_ed25519']),
+            d['circ'], d['dest_url'], d['scanner'],
+            msg=d['msg'], t=d['time'])
+
+    def to_dict(self):
+        d = super().to_dict()
+        return d
+
+
+class ResultErrorDestination(ResultError):
+    """
+    Error when there is not a working destination Web Server.
+
+    It is instanciated in :func:`~sbws.core.scanner.measure_relay`.
+
+    .. note:: this duplicates code and add more tech-debt,
+       since it's the same as the other
+       :class:`~sbws.lib.resultdump.ResultError` classes except for the
+       ``type``.
+       In a future refactor, there should be only one ``ResultError`` class
+       and assign the type in the ``scanner`` module.
+    """
+    def __init__(self, *a, **kw):
+        super().__init__(*a, **kw)
+
+    @property
+    def type(self):
+        return _ResultType.ErrorSecondRelay
+
+    @staticmethod
+    def from_dict(d):
+        assert isinstance(d, dict)
+        return ResultErrorSecondRelay(
             Result.Relay(
                 d['fingerprint'], d['nickname'], d['address'],
                 d['master_key_ed25519']),
@@ -445,7 +653,14 @@ class ResultErrorAuth(ResultError):
         return ResultErrorAuth(
             Result.Relay(
                 d['fingerprint'], d['nickname'], d['address'],
-                d['master_key_ed25519']),
+                d['master_key_ed25519'],
+                relay_in_recent_consensus_count=  # noqa
+                    d.get('relay_in_recent_consensus_count', None),  # noqa
+                relay_recent_measurement_attempt_count=  # noqa
+                    d.get('relay_recent_measurement_attempt_count', None),  # noqa
+                relay_recent_priority_list_count=  # noqa
+                    d.get('relay_recent_priority_list_count', None),  # noqa
+                ),
             d['circ'], d['dest_url'], d['scanner'],
             msg=d['msg'], t=d['time'])
 
@@ -480,7 +695,16 @@ class ResultSuccess(Result):
             Result.Relay(
                 d['fingerprint'], d['nickname'], d['address'],
                 d['master_key_ed25519'], d['relay_average_bandwidth'],
-                d['relay_observed_bandwidth']),
+                d.get('relay_burst_bandwidth'), d['relay_observed_bandwidth'],
+                d.get('consensus_bandwidth'),
+                d.get('consensus_bandwidth_is_unmeasured'),
+                relay_in_recent_consensus_count=  # noqa
+                    d.get('relay_in_recent_consensus_count', None),  # noqa
+                relay_recent_measurement_attempt_count=  # noqa
+                    d.get('relay_recent_measurement_attempt_count', None),  # noqa
+                relay_recent_priority_list_count=  # noqa
+                    d.get('relay_recent_priority_list_count', None),  # noqa
+                ),
             d['circ'], d['dest_url'], d['scanner'],
             t=d['time'])
 
@@ -490,7 +714,11 @@ class ResultSuccess(Result):
             'rtts': self.rtts,
             'downloads': self.downloads,
             'relay_average_bandwidth': self.relay_average_bandwidth,
+            'relay_burst_bandwidth': self.relay_burst_bandwidth,
             'relay_observed_bandwidth': self.relay_observed_bandwidth,
+            'consensus_bandwidth': self.consensus_bandwidth,
+            'consensus_bandwidth_is_unmeasured':
+                self.consensus_bandwidth_is_unmeasured,
         })
         return d
 
@@ -498,13 +726,11 @@ class ResultSuccess(Result):
 class ResultDump:
     ''' Runs the enter() method in a new thread and collects new Results on its
     queue. Writes them to daily result files in the data directory '''
-    def __init__(self, args, conf, end_event):
+    def __init__(self, args, conf):
         assert os.path.isdir(conf.getpath('paths', 'datadir'))
-        assert isinstance(end_event, Event)
         self.conf = conf
         self.fresh_days = conf.getint('general', 'data_period')
         self.datadir = conf.getpath('paths', 'datadir')
-        self.end_event = end_event
         self.data = {}
         self.data_lock = RLock()
         self.thread = Thread(target=self.enter)
@@ -534,21 +760,54 @@ class ResultDump:
         assert isinstance(result, Result)
         fp = result.fingerprint
         nick = result.nickname
-        if isinstance(result, ResultError) and self.end_event.is_set():
+        if isinstance(result, ResultError) and settings.end_event.is_set():
             log.debug('Ignoring %s for %s %s because we are shutting down',
                       type(result).__name__, nick, fp)
             return
         self.store_result(result)
         write_result_to_datadir(result, self.datadir)
-        log.info('%s %s finished measurement with %s', nick, fp[0:8],
-                 type(result).__name__)
+        if result.type == "success":
+            msg = "Success measuring {} ({}) via circuit {} and " \
+                  "destination {}".format(
+                    result.fingerprint, result.nickname, result.circ,
+                    result.dest_url)
+        else:
+            msg = "Error measuring {} ({}) via circuit {} and " \
+                  "destination {}: {}".format(
+                    result.fingerprint, result.nickname, result.circ,
+                    result.dest_url, result.msg)
+        # When the error is that there are not more functional destinations.
+        if result.type == "error-destination":
+            log.info("Shutting down because there are not functional "
+                     "destinations.")
+            # NOTE: Because this is executed in a thread, stop_threads can not
+            # be call from here, it has to be call from the main thread.
+            # Instead set the singleton end event, that will call stop_threads
+            # from the main process.
+            settings.end_event.set()
+        log.info(msg)
 
     def enter(self):
-        ''' Main loop for the ResultDump thread '''
+        """Main loop for the ResultDump thread.
+
+        When there are results in the queue, queue.get will get them until
+        there are not anymore or timeout happen.
+
+        For every result it gets, it process it and store in the filesystem,
+        which takes ~1 millisecond and will not trigger the timeout.
+        It can then store in the filesystem ~1000 results per second.
+
+        I does not accept any other data type than Results or list of Results,
+        therefore is not possible to put big data types in the queue.
+
+        If there are not any results in the queue, it waits 1 second and checks
+        again.
+
+        """
         with self.data_lock:
             self.data = load_recent_results_in_datadir(
                 self.fresh_days, self.datadir)
-        while not (self.end_event.is_set() and self.queue.empty()):
+        while not (settings.end_event.is_set() and self.queue.empty()):
             try:
                 event = self.queue.get(timeout=1)
             except Empty:
