@@ -10,7 +10,9 @@ from string import Template
 from tempfile import NamedTemporaryFile
 from sbws.globals import (DEFAULT_CONFIG_PATH, DEFAULT_LOG_CONFIG_PATH,
                           USER_CONFIG_PATH, SUPERVISED_RUN_DPATH,
-                          SUPERVISED_USER_CONFIG_PATH, fail_hard)
+                          SUPERVISED_USER_CONFIG_PATH)
+
+from sbws.util.iso3166 import ISO_3166_ALPHA_2
 
 _ALPHANUM = 'abcdefghijklmnopqrstuvwxyz'
 _ALPHANUM += _ALPHANUM.upper()
@@ -66,12 +68,18 @@ def _get_user_config(args, conf=None):
         assert isinstance(conf, ConfigParser)
     if args.config:
         if not os.path.isfile(args.config):
-            fail_hard('Configuration file %s not found.', args.config)
+            # XXX: The logger is not configured at this stage,
+            # sbws should start with a logger before reading configurations.
+            print('Configuration file %s not found, using defaults.' %
+                  args.config)
+            return conf
+        print('Using configuration provided as argument %s' % args.config)
         return _extend_config(conf, args.config)
     user_config_path = _obtain_user_conf_path()
     if os.path.isfile(user_config_path):
+        print('Using configuration file %s' % user_config_path)
         return _extend_config(conf, user_config_path)
-    log.debug('No user config found.')
+    log.debug('No user config found, using defaults.')
     return conf
 
 
@@ -117,8 +125,6 @@ def _can_log_to_file(conf):
 def configure_logging(args, conf):
     assert isinstance(conf, ConfigParser)
     logger = 'logger_sbws'
-    if args.log_level:
-        conf['logging']['level'] = args.log_level.upper()
     # Set the correct handler(s) based on [logging] options
     handlers = set()
     can_log_to_file, reason = _can_log_to_file(conf)
@@ -162,6 +168,15 @@ def configure_logging(args, conf):
         conf['logging']['to_stdout_level'].upper()
     conf['handler_to_syslog']['level'] = \
         conf['logging']['to_syslog_level'].upper()
+    # If there's a log_level cli argument, the user would expect that level
+    # in the standard output.
+    # conf['logging']['level'] sets the lower level, but it's still needed to
+    # set the stdout level.
+    # It also must be set up in the end, since cli arguments have higher
+    # priority.
+    if args.log_level:
+        conf['logging']['level'] = args.log_level.upper()
+        conf['handler_to_stdout']['level'] = conf['logging']['level']
     # Now we configure the standard python logging system
     with NamedTemporaryFile('w+t') as fd:
         conf.write(fd)
@@ -256,6 +271,21 @@ def _validate_paths(conf):
     return errors
 
 
+def _validate_country(conf, sec, key, err_tmpl):
+    errors = []
+    if conf[sec].get(key, None) is None:
+        errors.append(err_tmpl.substitute(
+            sec=sec, key=key, val=None,
+            e="Missing country in configuration file."))
+        return errors
+    valid = conf[sec]['country'] in ISO_3166_ALPHA_2
+    if not valid:
+        errors.append(err_tmpl.substitute(
+            sec=sec, key=key, val=conf[sec][key],
+            e="Not a valid ISO 3166 alpha-2 country code."))
+    return errors
+
+
 def _validate_scanner(conf):
     errors = []
     sec = 'scanner'
@@ -275,7 +305,7 @@ def _validate_scanner(conf):
         'download_max': {'minimum': 0.001, 'maximum': None},
     }
     all_valid_keys = list(ints.keys()) + list(floats.keys()) + \
-        ['nickname']
+        ['nickname', 'country']
     errors.extend(_validate_section_keys(conf, sec, all_valid_keys, err_tmpl))
     errors.extend(_validate_section_ints(conf, sec, ints, err_tmpl))
     errors.extend(_validate_section_floats(conf, sec, floats, err_tmpl))
@@ -283,6 +313,7 @@ def _validate_scanner(conf):
     if not valid:
         errors.append(err_tmpl.substitute(
             sec=sec, key='nickname', val=conf[sec]['nickname'], e=error_msg))
+    errors.extend(_validate_country(conf, sec, 'country', err_tmpl))
     return errors
 
 
@@ -375,15 +406,18 @@ def _validate_destinations(conf):
     urls = {
         'url': {},
     }
-    all_valid_keys = list(urls.keys()) + ['verify']
+    all_valid_keys = list(urls.keys()) \
+        + ['verify', 'country', 'max_num_failures']
     for sec in dest_sections:
         if sec not in conf:
             errors.append('{} is an enabled destination but is not a '
                           'section in the config'.format(sec))
             continue
         errors.extend(_validate_section_keys(
-            conf, sec, all_valid_keys, err_tmpl, allow_missing=['verify']))
+            conf, sec, all_valid_keys, err_tmpl,
+            allow_missing=['verify', 'max_num_failures']))
         errors.extend(_validate_section_urls(conf, sec, urls, err_tmpl))
+        errors.extend(_validate_country(conf, sec, 'country', err_tmpl))
     return errors
 
 
@@ -513,12 +547,16 @@ def _validate_enum(section, key, choices):
 
 def _validate_url(section, key):
     value = section[key]
-    if not value.startswith(('http://', 'https://')):
-        return False, 'Must start with http:// or https://'
     url = urlparse(value)
-    assert url.scheme in ['http', 'https']
     if not url.netloc:
         return False, 'Does not appear to contain a hostname'
+    # It should be possible to have an URL that starts by http:// that uses
+    # TLS,but python requests is just checking the scheme starts by https
+    # when verifying certificate:
+    # https://github.com/requests/requests/blob/master/requests/adapters.py#L215  # noqa
+    # When the scheme is https but the protocol is not TLS, requests will hang.
+    if url.scheme != 'https' and not url.netloc.startswith('127.0.0.1'):
+        return False, 'URL scheme must be HTTPS (except for the test server)'
     return True, ''
 
 
